from astroquery.astrometry_net import AstrometryNet
from astroquery.exceptions  import TimeoutError
from astropy.wcs import WCS
import numpy as np
from scipy.ndimage import median_filter
from skimage.feature import blob_dog, blob_log
import skimage.exposure as skie

ast = AstrometryNet()
ast.api_key = 'jefjaaeekvrszphp'

__all__ = ['plate_solve', 'starSeeker']

def  plate_solve(data, image_file_path, write_fits = False, write_path = '', write_name = ''):

        """
        plate_solve takes fits image file data and the corresponding file string to the data and calls nova.astrometry.net to obtain and then integrate WCS into the HDU.

        Parameters
        ----------
        data: CCDdata
                CCDdata of the image to be solved.
        image_file_path: string
                Path to the image to be solved.
        write_fits: Boolean
                Whether to save the resulting data.
        write_path: string
                File path to save the resulting data under. Default is current directory.
        write_name: string
                File name to save the resulting data under.
        Returns
        -------
        wcs_hdu: CCDdata
                The CCDdata file with a wcs header.
        """

        try_again = True
        submission_id = None
        num = 0

        while try_again:
                try:
                        if not submission_id:
                                wcs_header = ast.solve_from_image(image_file_path, force_image_upload=True, submission_id=submission_id, solve_timeout=300)
                        else:
                                print('Monitoring: try #', num)
                                wcs_header = ast.monitor_submission(submission_id, solve_timeout=120)
                except TimeoutError as e:
                        print(TimeoutError)
                        num = num + 1
                        print('Timed out: try #', num)
                        submission_id = e.args[1]


                if wcs_header != None:
                        # got a result, so terminate while loop
                        try_again = False
        if wcs_header:
                # Code to execute when solve succeeds
                print('Solve succeeded! :)')
                # print(wcs_header.tostring( sep='\n', endcard=True, padding=True))  # items, keys, values
                wcs_hdu = data
                wcs_hdu.header = wcs_header
                if write_fits:
                        if write_name != '':
                                wcs_hdu.write(write_path + write_name + '.fits', overwrite = True)
                        else:
                                print('The name to write this file was not provided. File saved under \'dorado_wcs.fits\'')
                                wcs_hdu.write(write_path + 'dorado_wcs.fits', overwrite = True)
                return wcs_hdu
        else:
                # Code to execute when solve fails
                print('Solve failed! :(')
                return 


def starSeeker(data):
        mf = median_filter(data, size= 15)
        datamf = data - mf
        limg = np.arcsinh(datamf) #datamf
        limg = limg / limg.max()
        low = np.percentile(limg, 0.2)
        high = np.percentile(limg, 99.1)
        opt_img  = skie.exposure.rescale_intensity(limg, in_range=(low,high))

        stars =  blob_log(opt_img, max_sigma=100, min_sigma = 5, num_sigma=10, threshold=.2)
        # stars =  blob_dog(opt_img, max_sigma=30, threshold=.2)

        # Compute radii in the 3rd column.
        stars[:, 2] = stars[:, 2] * np.sqrt(2)

        y2, x2, r = stars[:, 0], stars[:, 1], stars[:, 2]


        print('\nStars found: ', len(r))

        return x2, y2, r, opt_img




import dorado
import astroalign as aa
import datetime

## One might be interested in how long this all takes
START_DATE_TIME = datetime.datetime.now()
print('\nStarting time: ', START_DATE_TIME)
print('\nFinding data...')



## Organize and prepare your data
# Get a fits file formated date string for the current observing night.
night = dorado.get_night() 
print('Night: ', night)
# Your data directory
# Set night as directory 
directory = 'C:/Data/' + night

# You can name your target object
target = 'WASP33b'

bias_list, flats_list, lights_list = dorado.checkdir(directory + '/wrk/calibrated/')

# Read the data in
series = dorado.get_series(directory + '/wrk/calibrated/', lights_list[0:200], unit = None)

CAL_END_DATE_TIME = datetime.datetime.now()
print('\nFinished time: ', CAL_END_DATE_TIME)
print("Time elapsed for Calibration: ", (CAL_END_DATE_TIME - START_DATE_TIME))
print('\n\n\n\n')

## Plate solve your data series
# Pick image to solve and store its data
psimg = series[0]
# Set up a file string for dorado to locate image
expath = '/wrk/calibrated/' 
im = target +  '-0_c.fit'
image_file_path = dorado.file_string(directory + expath, im)
# Pass the data and file string to dorado
solved = dorado.plate_solve(psimg, image_file_path, write_fits = True, write_path = directory + '/wrk/', write_name = target + '_solved')
PLATE_END_DATE_TIME = datetime.datetime.now()
print('\nFinished time: ', PLATE_END_DATE_TIME)
print("Time elapsed for Plate solving: ", (PLATE_END_DATE_TIME - CAL_END_DATE_TIME))
print("Time elapsed so far: ", (PLATE_END_DATE_TIME - START_DATE_TIME))
print('\n\n\n\n')





# Align each image with the plate solved image
aa_series = []
for image in series:
    img_r, footprint_r = aa.register(image.data, solved.data)
    aa_series.append(img_r)

dorado.write_series(directory + '/wrk/aligned/', aa_series, target, '_a')
# You now have a fully calibrated and aligned data set with WCS data
AL_END_DATE_TIME = datetime.datetime.now()
print('\nFinished time: ', AL_END_DATE_TIME)
print("Time elapsed for Aligning: ", (AL_END_DATE_TIME - PLATE_END_DATE_TIME))
print("Time elapsed so far: ", (AL_END_DATE_TIME - START_DATE_TIME))
print('\n\n\n\n')



import warnings
warnings.filterwarnings('ignore')

import os
import datetime
from astropy.nddata import CCDData
from astropy.io import fits
import astropy.units as u
import numpy as np

__all__ = ['mkwrkdir', 'get_night', 'checkdir', 'get_series', 'write_series', 'get_im', 'file_string']

def mkwrkdir(datadir):
        """
        mkwrkdir creates standard data processing working directories.
        
        Parameters
        ----------
        datadir: string
                Path to data directory to make working folder.

        Returns
        -------
        None
        """
        # Allow for directories to already exist: This should be working, yet to test.
        # night, expath = ''
        stardir = os.getcwd()

        path = datadir # + night + '/' + expath # + '/'
        os.chdir(path)
        os.makedirs(path + '/wrk', exist_ok = True)
        path = path + '/wrk'
        os.makedirs(path + '/bias', exist_ok = True)
        os.makedirs(path + '/flats', exist_ok = True)
        os.makedirs(path + '/darks', exist_ok = True)
        os.makedirs(path + '/lights', exist_ok = True)
        os.makedirs(path + '/calibrated', exist_ok = True)
        os.makedirs(path + '/aligned', exist_ok = True)
        # os.mkdir(path + '/stars')
        os.makedirs(path + '/results', exist_ok = True)

        os.chdir(stardir)

def get_night():
        """
        get_night obtains a timestring for the most recent(previous) night.

        Parameters
        ----------
        None

        Returns
        -------
        night: string
                Timestring for the most recent night.
        """
        # currently does not support first/last of the month
        date = datetime.date.today()
        year = date.year
        month = date.month
        date2 = date.day
        date1 = date2 - 1
        night = str(year) + '-' + str(month) + '-' + str(date1) + '+' + str(date2)
        return night

def checkdir(directory):
        """
        checkdir returns lists

        Parameters
        ----------
        directory: string
                Path to data directory to make working folder.

        Returns
        -------
        bias, flats, darks, lights: arrays
                Arrays containing the filestrings for the bias, flats, and lights.
        """ 

        # modify to set bias, flat, dark, and light naming conventions

        path = directory # + '/' + night + '/' + expath
        print(path)
        dirlist = os.listdir(path)

        bias = [s for s in dirlist if 'BIAS' in s]
        if len(bias)==0:
                bias = [s for s in dirlist if 'Bias' in s]
        if len(bias)==0:
                bias = [s for s in dirlist if 'bias' in s]

        
        flats = [s for s in dirlist if 'FLAT' in s]
        if len(flats)==0:
                flats = [s for s in dirlist if 'FlatField' in s]
        if len(flats)==0:
                flats = [s for s in dirlist if 'flat' in s] # add this to lights catch


        darks = [s for s in dirlist if 'DARK' in s]
        if len(darks)==0:
                darks = [s for s in dirlist if 'Dark' in s]
        if len(darks)==0:
                darks = [s for s in dirlist if 'dark' in s] # add this to lights catch

        lights = [s for s in dirlist if (np.invert('FLAT' in s)) and (np.invert('Flat' in s)) and (np.invert('BIAS' in s)) and (np.invert('Bias' in s)) and (np.invert('DARK' in s)) and (np.invert('Dark' in s)) and ('.FIT' in s) ]
        if len(lights)==0:
                lights = [s for s in dirlist if (np.invert('FLAT' in s)) and (np.invert('Flat' in s)) and (np.invert('BIAS' in s)) and (np.invert('Bias' in s)) and (np.invert('DARK' in s)) and (np.invert('Dark' in s)) and ('.fit' in s) ]
        if len(lights)==0:
                lights = [s for s in dirlist if (np.invert('FLAT' in s)) and (np.invert('Flat' in s)) and (np.invert('BIAS' in s)) and (np.invert('Bias' in s)) and (np.invert('DARK' in s)) and (np.invert('Dark' in s)) and ('.fits' in s) ]
        if len(lights)==0:
                lights = [s for s in dirlist if (np.invert('FLAT' in s)) and (np.invert('Flat' in s)) and (np.invert('BIAS' in s)) and (np.invert('Bias' in s)) and (np.invert('DARK' in s)) and (np.invert('Dark' in s)) and ('.FITS' in s) ]

        print('\ndirlist: ', len(dirlist))
        print('\nbias\': ', len(bias))
        print('\nflats: ', len(flats))
        print('\ndarks: ', len(darks))
        print('\nlights: ', len(lights))
        print('\ntotal: ', len(lights) + len(flats) + len(bias))

        return bias, flats, darks, lights

def get_series(directory, imlist, unit = u.adu):
        """
        plate_solve takes fits image file data and the corresponding file string to the data and calls nova.astrometry.net to obtain and then integrate WCS into the HDU.

        Parameters
        ----------
        directory: filestring
                Path to data directory containing files.
        imlist: array[string]
                array of filestrings for images.
        unit: astropy.unit
                unit for the CCDdata.
        Returns
        -------
        series: array[CCDdata]
                The CCDdata.
        """
        # directory, night, imlist, expath = '', unit = u.adu
        ## rewrite to match get series in reduce_series
        stardir = os.getcwd()
        # print(stardir)
        path = directory # + '/' + night + '/' + expath
        # print(path)
        os.chdir(path)
        series = []
        for i in range(len(imlist)):
                # os.chdir(path)
                hdu = CCDData.read(imlist[i], unit=unit)
                series.append(hdu)
        # series = ccdproc.ImageFileCollection(filenames=imlist)
        os.chdir(stardir)


        return series

def write_series(directory, series, target, expath = ''):
        """
        write_series intakes a series of CCD images, a target name for filestring 
        creation and a data directory path and writes the series to the directory.
        
        Parameters
        ----------
        directory: filestring
                Path to the directory.
        series: array[CCDdata]
                Data series to write.
        target: String
                Target name to use in the file name prefix.
        expath: String
                Suffix to use in file saving. For example '_c' for calibrated
                or '_a' for aligned.

        Returns
        -------
        None
        """
        # add ability to specify file labels.

        #directory, night, series, target
        stardir = os.getcwd()
        # caldir = directory + night + '/wrk/calibrated'
        os.chdir(directory)
        for i in range(len(series)):
                series[i].write(target + '_' + str(i) + expath + '.fit', overwrite = True)


        os.chdir(stardir)

def get_im(directory, imname):
        """
        get_im intakes a filename and directory of a fits image and returns the 
        data and header of the image.
        To be depreciated.

        Parameters
        ----------
        directory: filestring
                Path to the image.
        imname: filestring
                Image filestring.
        Returns
        -------
        hdu_data: hdu.data
                The fits image data.
        hdu_header: hdu.data
                The fits image data.
        """
        # unify with get_series
        os.chdir(directory)
        file_string = imname
        hdu_data = fits.getdata(file_string)
        hdu_header = fits.getheader(file_string)

        return hdu_data, hdu_header

def file_string(directory, imname):
        """
        file_string takes a directory filestring and image filename and combines 
        them into a single filestring.
        To be depreciated.

        Parameters
        ----------
        directory: filestring
                Path to the image.
        imname: filestring
                Image filestring.
        Returns
        -------
        path: filestring
                combined filestring.
        """
        # directory, night, im, expath = ''
        # this is a temporary function for ease of testing
        path = directory + imname
         # path = directory + '/' + night + '/' + expath + im
        return path

# def lightcurve_save
# def timing/logging


import dorado
import astroalign as aa


## Organize and prepare your data
# Get a fits file formated date string for the current observing night.
night = dorado.get_night() 
print(night)
# Your data directory
# Set night as directory 
directory = 'C:/Data/' + night

# You can name your target object
target = 'dorado'

# Create a working directory
dorado.mkwrkdir(directory)


# Catalogue input data from the data directory
bias_list, flats_list, lights_list = dorado.checkdir(directory)





## Calibrate your data
# Read the data in
data_series = dorado.get_series(directory, lights_list)
flats_series = dorado.get_series(directory, flats_list)
bias_series = dorado.get_series(directory, bias_list)

# Produce master reduction images
bias = dorado.mastBias(directory, bias_list)
flat = dorado.mastFlat(directory, flats_list, bias)

# Calibrate data series
series = dorado.reduce_series(directory, lights_list, flat, bias, target)

# Write calibrated data
dorado.write_series(directory, series, target)





## Plate solve your data series
# Pick image to solve and store its data
data = series[0]
# Set up a file string for dorado to locate image
expath = '/wrk/calibrated/' 
im = target +  '-0_c.fit'
image_file_path = dorado.file_string(directory + expath, im)
# Pass the data and file string to dorado
solved = dorado.plate_solve(data, image_file_path, write_fits = True, write_name = target + '_solved')




# Align each image with the plate solved image
aa_series = []
for image in series:
    img_r, footprint_r = aa.register(image.data, solved.data)
    aa_series.append(img_r)

dorado.write_series(directory + '/wrk/aligned/', aa_series, target + '_a')
# You now have a fully calibrated and aligned data set with WCS data












# exotest.py
import dorado
import astroalign as aa
import datetime
import numpy as np
from astroquery.simbad import Simbad
from astropy.nddata import CCDData
from astropy.table import QTable, Table
from astropy.io import fits
from astropy import units as un

# time imports
from astropy.time import Time
from astropy.timeseries import TimeSeries

# photometry imports
from photutils.psf import IntegratedGaussianPRF, DAOGroup
from photutils.background import MMMBackground, MADStdBackgroundRMS
from astropy.modeling.fitting import LevMarLSQFitter
from astropy.stats import gaussian_sigma_to_fwhm
from photutils.psf import IterativelySubtractedPSFPhotometry
from photutils import DAOStarFinder
from astropy.stats import mad_std

# coordinate imports
from astropy.wcs import WCS
from astropy.coordinates import SkyCoord as acoord
from astropy.coordinates import Distance


## One might be interested in how long this all takes
START_DATE_TIME = datetime.datetime.now()
print('\nStarting time: ', START_DATE_TIME)
print('\nFinding data...')





## Organize and prepare your data
directory = 'C:/Data/2020-22-04+05/'
# You can name your target object
target = 'WASP33b'
s = Simbad()
r = s.query_object(target)
r.pprint()
print(r.colnames)
co = acoord(ra = r['RA'], dec = r['DEC'], unit = (un.hourangle, un.deg), frame = 'icrs')

# Create a working directory
dorado.mkwrkdir(directory)


# Catalogue input data from the data directory
_, _, _, lights_list = dorado.checkdir(directory + 'lights/')
_, flats_list, _, _= dorado.checkdir(directory + 'flats/')
bias_list, _, _, _ = dorado.checkdir(directory + 'bias/')



## Calibrate your data
# Read the data in
# data_series = dorado.get_series(directory + 'lights/', lights_list)
# flats_series = dorado.get_series(directory, flats_list)
# bias_series = dorado.get_series(directory, bias_list)

# Produce master reduction images
bias = dorado.mastBias(directory + 'bias/', bias_list, caldir = directory)
flat = dorado.mastFlat(directory + 'flats/', flats_list, bias, caldir = directory)

# Calibrate data series, files will be written automatically
series = dorado.reduce_series(directory + 'lights/', lights_list, flat, bias, target, caldir = directory)

CAL_END_DATE_TIME = datetime.datetime.now()
print('\nFinished time: ', CAL_END_DATE_TIME)
print("Time elapsed for calibration: ", (CAL_END_DATE_TIME - START_DATE_TIME))
print('\n\n\n\n')





## Plate solve your data series
# Pick image to solve and store its data
psimg = series[0]
# Set up a file string for dorado to locate image
expath = '/wrk/calibrated/' 
im = target +  '-0_c.fit'
# image_file_path = dorado.file_string(directory + expath, im)
image_file_path = dorado.file_string(directory, im)

# Pass the data and file string to dorado
solved = dorado.plate_solve(psimg, image_file_path, write_fits = True, write_name = target + '_solved')
w = WCS(solved.header)
PLATE_END_DATE_TIME = datetime.datetime.now()
print('\nFinished time: ', PLATE_END_DATE_TIME)
print("Time elapsed for solving: ", (PLATE_END_DATE_TIME - CAL_END_DATE_TIME))
print("Time elapsed so far: ", (PLATE_END_DATE_TIME - START_DATE_TIME))
print('\n\n\n\n')





# Align each image with the plate solved image
aa_series = []
for image in series:
    img_r, footprint_r = aa.register(image.data, solved.data)
    image.data = img_r
    aa_series.append(image)

dorado.write_series(directory + '/wrk/aligned/', aa_series, target, '_a')
# You now have a fully calibrated and aligned data set with WCS data
AL_END_DATE_TIME = datetime.datetime.now()
print('\nFinished time: ', AL_END_DATE_TIME)
print("Time elapsed for alignment: ", (AL_END_DATE_TIME - PLATE_END_DATE_TIME))
print("Time elapsed so far: ", (AL_END_DATE_TIME - START_DATE_TIME))
print('\n\n\n\n')





## Data analysis
# Run PSF photometry on image one
sigma_psf = 2.0
bkgrms = MADStdBackgroundRMS()
std = bkgrms(aa_series[0])
bkg_sigma = mad_std(aa_series[0]) 
daofind = DAOStarFinder(fwhm = 4., threshold = 3. * bkg_sigma)  
s = daofind(aa_series[0].data) 

daogroup = DAOGroup(2.0 * sigma_psf * gaussian_sigma_to_fwhm)
mmm_bkg = MMMBackground()
fitter = LevMarLSQFitter()
psf_model = IntegratedGaussianPRF(sigma = sigma_psf)

photometry = IterativelySubtractedPSFPhotometry(finder = daofind, 
                        group_maker = daogroup, bkg_estimator = mmm_bkg,
                         psf_model = psf_model, fitter = LevMarLSQFitter(),
                          niters = 1, fitshape = (11, 11))
result_tab = photometry(image = aa_series[0])

sources = Table()
sources['ra'], sources['dec'] = w.wcs_pix2world(result_tab['x_fit'], result_tab['y_fit'], 1) 
sources['x'] = result_tab['x_fit']
sources['y'] = result_tab['y_fit']
sources['flux'] = result_tab['flux_fit']
sources['flux_unc'] = result_tab['flux_unc']

# clean sources
sources = sources[sources['flux'] > 2000]
sources['mag_inst'] = -2.5 * np.log10(sources['flux'] / 15)
sources['mag_inst_unc'] = sources['flux_unc'] / ((sources['flux'] / 15) * 2.30258509)
sources.write('sources_test.csv', overwrite = True)
# fix positions to run PSF photometry on remaining images
target_px_x, target_px_y = w.wcs_pix2world(co)
print('target px: ', target_px_x, target_px_y)
target_s = sources[np.abs(sources[np.abs(sources['x'] - target_px_x) < 4]['y'] - target_px_y) < 4]
print('target in sources: ', target_s)

psf_model.x_0.fixed = True
psf_model.y_0.fixed = True
pos = Table(names=['x_0', 'y_0'], data=[sources['x'], sources['y']])

# ts = QTable(names=('time', 'flux', 'flux_unc', 'dmag', 'dmag_std', 'dmag_unc'))
ts = QTable(names=('time', 'dmag', 'dmag_std', 'dmag_unc'))


S1_END_DATE_TIME = datetime.datetime.now()
print('\nFinished time: ', S1_END_DATE_TIME)
print("Time elapsed for photometry 1: ", (S1_END_DATE_TIME - AL_END_DATE_TIME))
print("Time elapsed so far: ", (S1_END_DATE_TIME - START_DATE_TIME))
print('\n\n\n\n')

for image in aa_series:
    time = Time(image.header['DATE-OBS'], format = 'fits', scale = 'utc')

    std = bkgrms(image)
    psf_model = IntegratedGaussianPRF(sigma = sigma_psf)
    psf_model.x_0.fixed = True
    psf_model.y_0.fixed = True

    photometry = IterativelySubtractedPSFPhotometry(finder = daofind, 
                        group_maker = daogroup, bkg_estimator = mmm_bkg,
                         psf_model = psf_model, fitter = LevMarLSQFitter(),
                          niters = 1, fitshape = (11, 11))
    result_tab = photometry(image = image, init_guesses=pos)
    result_tab['mag'] = -2.5 * np.log10(result_tab['flux_fit'] / 15)
    result_tab['mag_unc'] = result_tab['flux_unc'] / ((result_tab['flux_fit'] / 15) * 2.30258509)

    target_idx = result_tab[np.abs(result_tab[np.abs(result_tab['x_fit'] - target_px_x) < 4]['y_fit'] - target_px_y) < 4]
    result_tab['dmag'] = target_idx['mag'] - result_tab['mag']
    dmag = np.mean(result_tab['dmag'])
    dmag_std  = np.std(result_tab['dmag'])
    dmag_unc = target_idx['mag_unc']
    ts.add_row((time, dmag, dmag_std, dmag_unc))

ts.show_in_browser(jsviewer = True)
ts.write('timeseries_test.csv', overwrite = True)


# Construct lightcurve

# Locate times of max light

# Call ephemeris to get O-C

## exit script, it took this long
END_DATE_TIME = datetime.datetime.now()
print('\nFinished time: ', END_DATE_TIME)
print("Time elapsed for run: ", (END_DATE_TIME - START_DATE_TIME))




# Finished time:  2021-01-01 19:47:59.883863
# Time elapsed for alignment:  0:11:47.918621
# Time elapsed so far:  0:16:19.697864





# Traceback (most recent call last):
#   File "c:/Users/mucep/OneDrive/Documents/GitHub/DORADO/dorado/examples/exotest.py", line 153, in <module>
#     sources['mag_inst'] = -2.5 * np.log10(sources['flux']/ aa_series[0].header['EXPTIME'])
#   File "C:\Users\mucep\AppData\Local\Packages\PythonSoftwareFoundation.Python.3.8_qbz5n2kfra8p0\LocalCache\local-packages\Python38\site-packages\astropy\io\fits\header.py", line 148, in __getitem__
#     card = self._cards[self._cardindex(key)]
#   File "C:\Users\mucep\AppData\Local\Packages\PythonSoftwareFoundation.Python.3.8_qbz5n2kfra8p0\LocalCache\local-packages\Python38\site-packages\astropy\io\fits\header.py", line 1708, in _cardindex
#     raise KeyError(f"Keyword {keyword!r} not found.")
# KeyError: "Keyword 'EXPTIME' not found."


import numpy as np
from scipy.interpolate import LSQUnivariateSpline, InterpolatedUnivariateSpline, UnivariateSpline
'''
Fournax is an abbreviation of Fourier numerical astronomy extension, its name is a backronym styled to match the constellation 'fornax'. 

A major future update will bring a more powerful curve fitting function to this sub-package.
'''

# __all__ = ['four_curve', 'pour', 'var_ephem']
# class Fournax():

def four_curve(y, terms):
        """
        Four_curve takes a timeseries and 'smoothes' out higher order terms in frequency space and returns the smoothed values.

        Parameters
        ----------
        y: list or array
                y values of a timeseries. Time is x.
        terms: int
                terms to retain in fourier series.
        Returns
        -------
        Fcurve: list or array
                Fourier smoothed y values
        """
        # Fourier series domain
        # Compute real valued Fourier transform
        f = np.fft.rfft(y)
        # Null or zero coefficients above ammount of series "terms"
        # This corresponds to undesired high-frequency terms
        f[terms+1:] = 0
        # Collapse back into function space, result is smoothed Fourier curve
        Fcurve = np.fft.irfft(f)
        
        return Fcurve

def pour(y):
        '''
        pour takes a timeseries and computes the power in frequency space.

        Parameters
        ----------
        y: list or array
                y values of a timeseries. Time is x.

        Returns
        -------
        p: list or array
                power of y

        '''
        # Compute real valued Fourier transform
        f = np.fft.fft(y)
        # p = np.square(np.abs(f))
        p = np.square(np.abs(f))
        
        return p

def var_ephem(OBS, epoch, period):
        '''
        var_ephem takes observed time(s) of max light for a repeating variable star and ephemeris data and returns O-C values as well as the corresponding cycle.

        Parameters
        ----------
        OBS: float, list, or array
                Observed time(s) of max light
        epoch: float
                Epoch of ephemeris
        period: float
                Period of ephemeris 
        Returns
        -------
        cycle: float, list, or array
                The cycle corresponding to the time(s) of max light
        OmC: float, list, or array
                The O-C value for the time(s) of max light
        '''
        cycle_ref = (OBS-epoch)/period
        cycle = np.round(cycle_ref)
        OmC = OBS-(epoch+cycle*period)
        return cycle, OmC

def superfit(x, y, terms, s):
        '''
        superfit takes a raw timeseries and performs a multistep smoothed fit of the data
        which includes a spline fit tailored with the curvature of knots from a spline
        fit of a Fourier fit. The result is then expanded and then convoluted with a
        'blackman' window function.
        Parameters
        ----------
        x, y: array
                timeseries arrays to be fit
        terms: int
                terms to retain in the fourier fit.
        s: int
                new data array size 
        Returns
        -------
        X, Y: array
                The superfit timeseries arrays.

        '''

        # Fourier fit the data to model curvature
        Fcurve = four_curve(y, terms)
        # Create a spline fit of the fourier fit to extract knots
        tispl = InterpolatedUnivariateSpline(x[:len(Fcurve)], Fcurve, k = 5)
        # feed knots into spline of raw data
        LSQspl = LSQUnivariateSpline(x, y, tispl.get_knots()[1:-1]) 
        X = np.linspace(np.min(x), np.max(x), s)

        Y = smooth(LSQspl(X), window = 'blackman')

        Y = Y[5:-5]

        return X, Y

def smooth(x,window_len=11,window='hanning'):
        """smooth the data using a window with requested size.

        This method is based on the convolution of a scaled window with the signal.
        The signal is prepared by introducing reflected copies of the signal 
        (with the window size) in both ends so that transient parts are minimized
        in the begining and end part of the output signal.

        input:
        x: the input signal 
        window_len: the dimension of the smoothing window; should be an odd integer
        window: the type of window from 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'
                flat window will produce a moving average smoothing.

        output:
        the smoothed signal

        example:

        t=linspace(-2,2,0.1)
        x=sin(t)+randn(len(t))*0.1
        y=smooth(x)

        see also: 

        numpy.hanning, numpy.hamming, numpy.bartlett, numpy.blackman, numpy.convolve
        scipy.signal.lfilter

        TODO: the window parameter could be the window itself if an array instead of a string
        NOTE: length(output) != length(input), to correct this: return y[(window_len/2-1):-(window_len/2)] instead of just y.

        https://scipy-cookbook.readthedocs.io/items/SignalSmooth.html
        2017-07-13 (last modified), 2006-10-31 (created)
        Section author: Unknown[1], GaelVaroquaux, Unknown[142], Unknown[143], Unknown[144], Unknown[145], Unknown[146], Unknown[147], WesTurner, Christian Gagnon, clecocel
        """
        if x.ndim != 1:
                raise ValueError("smooth only accepts 1 dimension arrays.")

        if x.size < window_len:
                raise ValueError("Input vector needs to be bigger than window size.")


        if window_len<3:
                return x


        if not window in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
                raise ValueError("Window is none of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'")


        s = np.r_[x[window_len-1:0:-1], x, x[-2:-window_len-1:-1]]
        # print(len(s))
        if window == 'flat': # moving average
                w = np.ones(window_len,'d')
        else:
                w = eval('np.' + window +'(window_len)')

        y = np.convolve(w/w.sum(), s, mode='valid')

        return y

# def maxlight
# def properties




from astroquery.simbad import Simbad
import numpy as np
from astropy import units as un
from astropy.coordinates import SkyCoord as acoord
from astropy.coordinates import Distance
from astropy.table import QTable, Table
from photutils.detection import IRAFStarFinder
from photutils import DAOStarFinder
from astropy.stats import mad_std
from astropy.wcs import WCS

from astropy.nddata import CCDData
from astropy.io import fits
from photutils.psf import IntegratedGaussianPRF, DAOGroup
from photutils.background import MMMBackground, MADStdBackgroundRMS
from astropy.modeling.fitting import LevMarLSQFitter
from astropy.stats import gaussian_sigma_to_fwhm
from photutils.psf import IterativelySubtractedPSFPhotometry

directory = 'E:/DORADO/2020-22-04+05/'

target = 'WASP33b'
s = Simbad()
r = s.query_object(target)
r.pprint()
print(r.colnames)
# print(r['RA'], r['DEC'])


class toi:
    # work on a class for targets of interest

    # initialization of class
    def __init__(self, name):
        # identification
        self.name = name # toi name
        self.id = None # toi ID
        self.object_type = None # object type of toi
        # celestial coordinates
        self.ra = None # right accension of toi
        self.dec = None # declination of toi
        self.distance = None # distance quantity of toi
        self.z = None # redshift of toi
        self.co = None # coordinate object of toi
        # initialize photometry
        self.flux = QTable(names=('filter', 'flux', 'unc'))
        self.mag = QTable(names=('filter', 'mag', 'unc'))
        # image coordinates 
        self.xpx = None # x coordinate of toi
        self.ypx = None # y coordinate of toi

    # class functions
    def add_flux(self, filt, flux, unc, add_mag = False):
        self.flux.add_row((filt, flux, unc))
        if add_mag:
            mag_unc = unc / (flux * 2.30258509)
            mag = -2.5 * np.log10(flux)
            self.add_mag(filt, mag, mag_unc)
                
    def add_mag(self, filt, mag, unc):
        self.mag.add_row((filt, mag, unc))

    def init_co(self, ra, dec, distance = None, z = None, frame = 'icrs', units = None):
        # initialize toi coordinates
        if distance != None:
            self.distance = Distance(distance)
            if units != None:
                self.co = acoord(ra = ra, dec = dec, frame = frame, distance = distance, unit = units)
            else:
                self.co = acoord(ra = ra, dec = dec, frame = frame, distance = distance)

            self.ra = self.co.ra
            self.dec = self.co.dec
            if z != None:
                self.z = z
            else:
                self.z = self.distance.z

        elif z != None:
            self.distance = Distance(z = z)
            if units != None:
                self.co = acoord(ra = ra, dec = dec, frame = frame, distance = distance, unit = units)
            else:
                self.co = acoord(ra = ra, dec = dec, frame = frame, distance = distance) 
            
            self.ra = self.co.ra
            self.dec = self.co.dec
            self.z = z

        else: 
            if units != None:
                self.co = acoord(ra = ra, dec = dec, frame = frame, unit = units)
            else:
                self.co = acoord(ra = ra, dec = dec, frame = frame)
            self.ra = self.co.ra
            self.dec = self.co.dec




wasp = toi(target)
wasp.init_co(ra = r['RA'], dec = r['DEC'], units = (un.hourangle, un.deg))
wasp.object_type = 'STAR'
# print(wasp.co.to_string('hmsdms'))


# read in test data
hdulist = fits.open(directory + 'wcs.fits')
# print(hdulist)
w = WCS(hdulist[0].header, hdulist)
hdulist.close()
axy = Table.read(directory + 'axy.fits')
image  = CCDData.read(directory + 'new-image.fits', unit = un.adu)

# set up PSF stuff
sigma_psf = 2.0
bkgrms = MADStdBackgroundRMS()
std = bkgrms(image)
iraffind = IRAFStarFinder(threshold = 3.5 * std, fwhm = sigma_psf * gaussian_sigma_to_fwhm,
                              minsep_fwhm = 0.01, roundhi = 5.0, roundlo = -5.0,
                               sharplo = 0.0, sharphi = 2.0)

bkg_sigma = mad_std(image) 
daofind = DAOStarFinder(fwhm = 4., threshold = 3. * bkg_sigma)  
s = daofind(image.data) 
# s.show_in_browser(jsviewer = True)

daogroup = DAOGroup(2.0 * sigma_psf * gaussian_sigma_to_fwhm)
mmm_bkg = MMMBackground()
fitter = LevMarLSQFitter()
psf_model = IntegratedGaussianPRF(sigma = sigma_psf)

# perform photometry
photometry = IterativelySubtractedPSFPhotometry(finder = daofind, 
                        group_maker = daogroup, bkg_estimator = mmm_bkg,
                         psf_model = psf_model, fitter = LevMarLSQFitter(),
                          niters = 1, fitshape = (11, 11))
# photometry = IterativelySubtractedPSFPhotometry(finder = iraffind, 
#                         group_maker = daogroup, bkg_estimator = mmm_bkg,
#                          psf_model = psf_model, fitter = LevMarLSQFitter(),
#                           niters = 1, fitshape = (11, 11))
result_tab = photometry(image = image)

sources = Table()
sources['ra'], sources['dec'] = w.wcs_pix2world(result_tab['x_fit'], result_tab['y_fit'], 1) 
sources['x'] = result_tab['x_fit']
sources['y'] = result_tab['y_fit']
sources['flux'] = result_tab['flux_fit']
sources['flux_unc'] = result_tab['flux_unc']

# clean sources
sources = sources[sources['flux'] > 2000]
sources['mag_inst'] = -2.5 * np.log10(result_tab['flux_fit'])
sources['mag_inst_unc'] = sources['flux_unc'] / (sources['flux'] * 2.30258509)
sources.show_in_browser(jsviewer = True)


# wasp33 B-V 0.297
# V mag 8.14

# dorado.plt_stars(image, sources['x'], sources['x'], rs)